# This code accompanies the paper "Santalo geometry of Convex Polytopes" by Dmitrii Pavlov and Simon Telen. 
# For each code snippet, we indicate the corresponding example in the manuscript. 
# To run this code, make sure that your working directory contains the folder Santalo. In the package manager, execute "activate Santalo". 

# Authors: Dmitrii Pavlov and Simon Telen
# Date: February 27, 2024

# Example 4.2: computing the adjoint polynomial in x-coordinates
using Santalo # load the package
A = [1 1 1 1 1; 2 1 0 1 0; 1 2 0 0 1]; b = 1//5*[5; 4; 4];
R, alpha = adjoint_x(A,b)
T, J = ideal_XC(A,b)

# Example 5.8: degree versus ML degree 
using LinearAlgebra
using Oscar
using HomotopyContinuation
for n = 3:11
    d = n-2
    # compute a totally positive matrix A
    A = Rational{BigInt}.(hcat([[(j//10)^i for i = 0:d-1] for j = 0:n-1]...)//1)
    A = A*diagm(rand(1:10,n))
    # the next lines compute a righthand side vector b in a maximal cell of the chamber complex
    P = [cone(transpose(A[:,setdiff(1:n,[i])])) for i = 1:n]
    V = collect(rays(intersect(P...)))
    V = [convert(Vector{Rational{BigInt}},v) for v in V]
    myb = sum(V)//n
    myu = randn(ComplexF64,size(A,2)+1)
    irredXCu, degXCu = degree_irred_XCuo(A,myb,myu)
    irredXC, degXC = degree_irred_XCuo(A,myb,ones(size(A,2)+1))
    S, monres = solve_likelihood_startsystem(A,myb)
    MLdeg = length(solutions(monres))
    println("n = $n, MLdeg = $MLdeg")
    if irredXCu && irredXC && degXC == degXCu
        println("all varieties irreducible of the same degree $degXC")
    else
        println("!!!!!!!! reducible variety or degree mismatch")
    end
    println("-------------------------------------------------")
end

# Example 6.1: computing Santalo points in y- and x-coordinates
using Santalo # load the package
A = [1 1 1 1 1; 2 1 0 1 0; 1 2 0 0 1] 
B = transpose(1//18*[5 -4 2 -6 3; -4 5 2 3 -6])
b = 1//5*[5; 4; 4]
Q = compute_Q(A,b,B) # Q = B^T*Pb
ystar = get_santalo_point(Q) # Santalo point in y-coordinates
xstar = get_santalo_point(A,b) # Santalo point in x-coordinates

# Example 6.2: the Santalo point of the 3D permutahedron
using Oscar # load the package Oscar to construct polytopes
Q = project_full(permutahedron(3))
ystar = get_santalo_point(Q) # output: (2.5, 2.5, 2.5)

A, b, W, c = free_representation(Q) 
solve_likelihood_startsystem(A,b)

# Example 6.3: tracking paths on the Santalo patchwork
A = [1 1 1 1 1; 2 1 0 1 0; 1 2 0 0 1]
b0 = 1//5*[5; 4; 4]; b1 = 1//5*[5; 5; 4];
x0 = get_santalo_point(A,b0)
x1 = santalo_path(A,b0,b1,x0) 





